package com.mortals.xhx.daemon.task;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.pdu.ListItem;
import com.mortals.xhx.common.pdu.NacosResponse;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Date;
import java.util.List;

/**
 * 检测项目运行状态
 */
@Slf4j
@Service("CheckProjectStatusTask")
public class CheckProjectStatusTaskImpl implements ITaskExcuteService {

    @Autowired
    private SetupProjectService setupProjectService;

    @Value("${project.nacosUrl}")
    private String nacosUrl;


    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("检测项目运行状态任务");

        List<SetupProjectEntity> setupProjectEntities = setupProjectService.find(new SetupProjectEntity());
        for (SetupProjectEntity setupProjectEntity : setupProjectEntities) {
            if (ProjectTypeEnum.后端.getValue() == setupProjectEntity.getProjectType()) {
                //todo http://192.168.0.250:8848/nacos/v1/ns/catalog/instances?message=true&serviceName=bill-manager&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov
                String serverUrl = nacosUrl + "/nacos/v1/ns/catalog/instances?message=true&serviceName=" + setupProjectEntity.getProjectCode() + "&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov";
                String resp = HttpUtil.get(serverUrl);

                try {
                    NacosResponse nacosResponse = JSON.parseObject(resp, new TypeReference<NacosResponse>() {
                    });
                    int count = nacosResponse.getCount();
                    if (count > 0) {
                        ListItem item = nacosResponse.getList().get(0);
                        boolean healthy = item.isHealthy();
                        if (healthy) {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                        } else {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                        }
                        setupProjectEntity.setUpdateTime(new Date());
                        setupProjectService.update(setupProjectEntity);
                    }

                    log.info("服务实例详细查询url:{},返回:{},naocsResponse:{}", serverUrl, resp, JSON.toJSONString(nacosResponse));

                } catch (JSONException e) {
                    log.info("json反序列化异常:{},返回:{}", e.getMessage(), resp);
                    if(setupProjectEntity.getProjectStatus()>ProjectStatusEnum.未部署.getValue()){
                        setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                        setupProjectEntity.setUpdateTime(new Date());
                        setupProjectService.update(setupProjectEntity);
                    }
                }
            } else if (ProjectTypeEnum.前端.getValue() == setupProjectEntity.getProjectType()) {
                //todo 前端 判断路径文件是否存在 如果存在 则代表运行
                String projectPath = setupProjectEntity.getProjectPath();
                String projectCode = setupProjectEntity.getProjectCode();
                String destPath = projectPath + File.separator + projectCode + File.separator + "dist" + File.separator + "index.html";
                File uiFile = new File(destPath);
                if (uiFile.exists()) {
                    setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                    setupProjectEntity.setUpdateTime(new Date());
                    setupProjectService.update(setupProjectEntity);
                } else {
                    setupProjectEntity.setProjectStatus(ProjectStatusEnum.未部署.getValue());
                    setupProjectEntity.setUpdateTime(new Date());
                    setupProjectService.update(setupProjectEntity);
                }
                log.info("前端资源路径:{},是否存在:{}", destPath, uiFile.exists());
            } else {

            }
        }


    }


    @Override
    public void stopTask(ITask task) throws AppException {

    }


}
