package com.mortals.xhx.module.setup.service.impl;

import cn.hutool.db.ds.simple.SimpleDataSource;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.druid.pool.DruidPooledConnection;
import com.alibaba.druid.proxy.jdbc.ConnectionProxy;
import com.alibaba.druid.util.JdbcUtils;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.service.SetupDbService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.FileSystemResource;
import org.springframework.core.io.support.EncodedResource;
import org.springframework.jdbc.datasource.init.ScriptUtils;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

@Service("SetupDbService")
@Slf4j
public class SetupDbServiceImpl implements SetupDbService {

    @Value("${project.publishPath}")
    private String publishPath;
    @Autowired
    private UploadService uploadService;

    private int timeout = 5;

    private String defaultValidateQuery = "SELECT 'x' FROM DUAL";

    @Override
    public Rest<String> initDb(DbSetupEntity dbSetupEntity) {
        try {
            //判断数据库是否已经存在 如果不存在 则链接默认库mysql 再初始化创建库
            DruidDataSource druidDataSource = new DruidDataSource();
            druidDataSource.setUrl("jdbc:mysql://" + dbSetupEntity.getDbHost() + ":" + dbSetupEntity.getDbPort() + "?useUnicode=true&characterEncoding=utf8&useSSL=false&serverTimezone=Hongkong");
            druidDataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
            druidDataSource.setUsername(dbSetupEntity.getUserName());
            druidDataSource.setPassword(dbSetupEntity.getPassword());

            DruidPooledConnection connection = druidDataSource.getConnection();
            boolean database = createDatabase(connection, dbSetupEntity.getDbName());
            if(database){
                connection.setCatalog(dbSetupEntity.getDbName());
                FileSystemResource rc = new FileSystemResource(uploadService.getFilePath(dbSetupEntity.getDbFilePath()));
                EncodedResource er = new EncodedResource(rc, "UTF-8");
                ScriptUtils.executeSqlScript(connection, er);
            }

        } catch (Exception e) {
            log.error("初始化数据库异常", e);

        }finally {

        }
        return Rest.ok("初始化数据成功！");
    }

    @Override
    public Rest<String> connect(DbSetupEntity dbSetupEntity) {
        try {
            SimpleDataSource simpleDataSource = new SimpleDataSource("jdbc:mysql://" + dbSetupEntity.getDbHost() + ":" + dbSetupEntity.getDbPort() + "?useUnicode=true&characterEncoding=utf8&useSSL=false&serverTimezone=Hongkong"
                    , dbSetupEntity.getUserName(), dbSetupEntity.getPassword(), "com.mysql.cj.jdbc.Driver");

            Connection connection = simpleDataSource.getConnection();

            boolean databaseExists = databaseExists(connection, dbSetupEntity.getDbName());
            if (!databaseExists) {
                throw new AppException("数据库不存在");
            }

            boolean valid = isValidConnection(connection, null, 0);
            if (valid) {
                return Rest.ok("连接数据库成功!");
            } else {
                throw new AppException("连接数据库失败!");
            }
        } catch (Exception e) {
            log.error("链接数据库异常", e);
            //return null;
            return Rest.fail(e.getMessage());
        }
    }

    @Override
    public Rest<DataSource> getDataSource(DbSetupEntity dbSetupEntity) {
        try {
            SimpleDataSource simpleDataSource = new SimpleDataSource("jdbc:mysql://" + dbSetupEntity.getDbHost() + ":" + dbSetupEntity.getDbPort() + "?useUnicode=true&characterEncoding=utf8&useSSL=false&serverTimezone=Hongkong"
                    , dbSetupEntity.getUserName(), dbSetupEntity.getPassword(), "com.mysql.cj.jdbc.Driver");

            return Rest.ok(simpleDataSource);
        } catch (Exception e) {
            log.error("链接数据库异常", e);
            return Rest.fail(e.getMessage());
        }
    }


    //创建数据库
    private boolean createDatabase(Connection conn, String dbName) throws SQLException {
        // 检查数据库是否已经存在
        if (!databaseExists(conn, dbName)) {
            Statement stmt = conn.createStatement();
            String sql = "CREATE DATABASE `" + dbName + "` default charset=utf8";
            stmt.executeUpdate(sql);
            log.info("成功创建数据库");
            stmt.close();
            return true;
        }
        log.info("数据库已经存在，无需创建与初始化");
        return false;

    }

    //检查bank数据库是否存在
    private boolean databaseExists(Connection conn, String dbName) throws SQLException {
        ResultSet resultSet = conn.getMetaData().getCatalogs();
        while (resultSet.next()) {
            if (dbName.equals(resultSet.getString(1))) {
                return true;
            }
        }
        return false;
    }


    //采用sql模式校验有效性
    public boolean isValidConnection(Connection conn, String validateQuery, int validationQueryTimeout) throws Exception {
        if (validateQuery == null || validateQuery.isEmpty()) {
            validateQuery = this.defaultValidateQuery;
        }

        if (conn.isClosed()) {
            return false;
        }

        if (conn instanceof DruidPooledConnection) {
            conn = ((DruidPooledConnection) conn).getConnection();
        }

        if (conn instanceof ConnectionProxy) {
            conn = ((ConnectionProxy) conn).getRawObject();
        }

        if (validateQuery == null || validateQuery.isEmpty()) {
            return true;
        }

        int queryTimeout = validationQueryTimeout <= 0 ? timeout : validationQueryTimeout;

        Statement stmt = null;
        ResultSet rs = null;
        try {
            stmt = conn.createStatement();
            stmt.setQueryTimeout(queryTimeout);
            rs = stmt.executeQuery(validateQuery);
            return true;
        } catch (Exception e) {
            throw new AppException(e.getMessage());
        } finally {
            JdbcUtils.close(rs);
            JdbcUtils.close(stmt);
        }
    }

}